#if !defined(BMPCTRL_H)
#define BMPCTRL_H

#include "bmpctrl_ifc.h"

class TImpIClassFactory : public IClassFactory
{
  protected:
      ULONG           m_cRef;

  public:
      TImpIClassFactory(void);
      ~TImpIClassFactory(void);

      //IUnknown members
      STDMETHODIMP         QueryInterface(REFIID, void**);
      STDMETHODIMP_(ULONG) AddRef(void);
      STDMETHODIMP_(ULONG) Release(void);

      //IClassFactory members
      STDMETHODIMP     CreateInstance(LPUNKNOWN, REFIID, void**);
      STDMETHODIMP     LockServer(BOOL);
};


//forward declarations
class TImpIOleObject;

//BMPCTRL.CPP
class TBmpCtrl : public IBmpCtrl
{
  friend class TImpIOleObject;

  private:
    ULONG             m_cRef;
    DWORD             m_dwConn;
    LPUNKNOWN         m_pDefIUnknown;    //IUnknown of Ole Data Cache
    LPUNKNOWN         m_pUnkOuter;       //IUnknown of outer aggregating object
    LPOLEOBJECT       m_pImpIOleObject;       //Aggregated
    LPOLECLIENTSITE   m_pIOleClientSite;      //From client
    LPOLEADVISEHOLDER m_pIOleAdviseHolder;    //From client
    LPTYPEINFO        m_pITINeutral;          
    LPUNKNOWN         m_pIUnkStdDisp;         //Standard Dispatch interface 
    CLSID             m_Clsid;
    void (*m_pfnDestroyed)();
    BSTR m_Title;


  public:
    TBmpCtrl(LPUNKNOWN, void(*pfnDestroyed)());
    ~TBmpCtrl();
    HRESULT Init();

    //IBmpCtrl methods
    BSTR STDMETHODCALLTYPE get_Title(void);
    void STDMETHODCALLTYPE put_Title(BSTR t); 
    HRESULT STDMETHODCALLTYPE GrabFromClipboard(void);

    //IUnknown members
    STDMETHODIMP         QueryInterface(REFIID, void**);
    STDMETHODIMP_(ULONG) AddRef(void);
    STDMETHODIMP_(ULONG) Release(void);
};
  

//IOLEOBJ.CPP
class TImpIOleObject: public IOleObject
{
  private:
      ULONG           m_cRef;
      TBmpCtrl*       m_pObj;
      LPUNKNOWN       m_pUnkOuter;

  public:
      TImpIOleObject(TBmpCtrl*, LPUNKNOWN);
      ~TImpIOleObject();

      //IUnknown members that delegate to m_pUnkOuter.
      STDMETHODIMP         QueryInterface(REFIID, void**);
      STDMETHODIMP_(ULONG) AddRef(void);
      STDMETHODIMP_(ULONG) Release(void);

      //IOleObject members
      STDMETHODIMP SetClientSite(LPOLECLIENTSITE);
      STDMETHODIMP GetClientSite(LPOLECLIENTSITE *);
      STDMETHODIMP SetHostNames(LPCOLESTR, LPCOLESTR);
      STDMETHODIMP Close(DWORD);
      STDMETHODIMP SetMoniker(DWORD, LPMONIKER);
      STDMETHODIMP GetMoniker(DWORD, DWORD, LPMONIKER *);
      STDMETHODIMP InitFromData(LPDATAOBJECT, BOOL, DWORD);
      STDMETHODIMP GetClipboardData(DWORD, LPDATAOBJECT *);
      STDMETHODIMP DoVerb(LONG, LPMSG, LPOLECLIENTSITE, LONG
		       , HWND, LPCRECT);
      STDMETHODIMP EnumVerbs(LPENUMOLEVERB *);
      STDMETHODIMP Update(void);
      STDMETHODIMP IsUpToDate(void);
      STDMETHODIMP GetUserClassID(CLSID *);
      STDMETHODIMP GetUserType(DWORD, LPOLESTR *);
      STDMETHODIMP SetExtent(DWORD, LPSIZEL);
      STDMETHODIMP GetExtent(DWORD, LPSIZEL);
      STDMETHODIMP Advise(LPADVISESINK, DWORD *);
      STDMETHODIMP Unadvise(DWORD);
      STDMETHODIMP EnumAdvise(LPENUMSTATDATA *);
      STDMETHODIMP GetMiscStatus(DWORD, DWORD *);
      STDMETHODIMP SetColorScheme(LPLOGPALETTE);
};

#endif //BMPCTRL_H
